@@ -18,6 +18,7 @@ import sys
 import warnings
 from collections import deque
 from os import path
+from typing import TYPE_CHECKING
 
 from docutils import nodes
 from docutils.parsers.rst import directives, roles
@@ -43,8 +44,7 @@ from sphinx.util.i18n import find_catalog_source_files
 from sphinx.util.osutil import ENOENT, ensuredir
 from sphinx.util.tags import Tags
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Tuple, Type, Union  # NOQA
     from docutils.parsers import Parser  # NOQA
     from docutils.transform import Transform  # NOQA
@@ -88,6 +88,7 @@ builtin_extensions = (
     'sphinx.roles',
     'sphinx.transforms.post_transforms',
     'sphinx.transforms.post_transforms.images',
+    'sphinx.util.compat',
     # collectors should be loaded by specific order
     'sphinx.environment.collectors.dependencies',
     'sphinx.environment.collectors.asset',
@@ -122,7 +123,7 @@ class Sphinx(object):
         self.enumerable_nodes = {}              # type: Dict[nodes.Node, Tuple[unicode, Callable]]  # NOQA
         self.html_themes = {}                   # type: Dict[unicode, unicode]
 
-        self.srcdir = srcdir
+        self.srcdir = srcdir                    # type: unicode
         self.confdir = confdir
         self.outdir = outdir
         self.doctreedir = doctreedir
