@@ -11,21 +11,21 @@
 
 import codecs
 from os import path
+from typing import TYPE_CHECKING
 
 from six import iteritems
 
 from sphinx import package_dir
+from sphinx.builders import Builder
 from sphinx.locale import _
 from sphinx.theming import HTMLThemeFactory
-from sphinx.builders import Builder
 from sphinx.util import logging
-from sphinx.util.osutil import ensuredir, os_path
 from sphinx.util.console import bold  # type: ignore
 from sphinx.util.fileutil import copy_asset_file
+from sphinx.util.osutil import ensuredir, os_path
 from sphinx.util.pycompat import htmlescape
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from typing import Any, Dict, List, Tuple  # NOQA
     from sphinx.application import Sphinx  # NOQA
 
@@ -38,6 +38,7 @@ class ChangesBuilder(Builder):
     Write a summary with all versionadded/changed directives.
     """
     name = 'changes'
+    epilog = 'The overview file is in %(outdir)s.'
 
     def init(self):
         # type: () -> None
@@ -73,8 +74,6 @@ class ChangesBuilder(Builder):
             ttext = self.typemap[type]
             context = content.replace('\n', ' ')
             if descname and docname.startswith('c-api'):
-                if not descname:
-                    continue
                 if context:
                     entry = '<b>%s</b>: <i>%s:</i> %s' % (descname, ttext,
                                                           context)
