@@ -5,26 +5,26 @@
 
     Build configuration file handling.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+import traceback
 from os import path, getenv
+from typing import TYPE_CHECKING, Any, NamedTuple, Union
 
 from six import PY2, PY3, iteritems, string_types, binary_type, text_type, integer_types
-from typing import Any, NamedTuple, Union
 
 from sphinx.errors import ConfigError
-from sphinx.locale import l_, _
+from sphinx.locale import l_, __
 from sphinx.util import logging
 from sphinx.util.i18n import format_date
 from sphinx.util.osutil import cd
 from sphinx.util.pycompat import execfile_, NoneType
 
-if False:
-    # For type annotation
-    from typing import Any, Callable, Dict, Iterable, Iterator, List, Tuple  # NOQA
+if TYPE_CHECKING:
+    from typing import Any, Callable, Dict, Iterable, Iterator, List, Tuple, Union  # NOQA
     from sphinx.util.tags import Tags  # NOQA
 
 logger = logging.getLogger(__name__)
@@ -35,6 +35,7 @@ copyright_year_re = re.compile(r'^((\d{4}-)?)(\d{4})(?=[ ,])')
 CONFIG_SYNTAX_ERROR = "There is a syntax error in your configuration file: %s"
 if PY3:
     CONFIG_SYNTAX_ERROR += "\nDid you change the syntax from 2.x to 3.x?"
+CONFIG_ERROR = "There is a programable error in your configuration file:\n\n%s"
 CONFIG_EXIT_ERROR = "The configuration file (or one of the modules it imports) " \
                     "called sys.exit()"
 CONFIG_ENUM_WARNING = "The config value `{name}` has to be a one of {candidates}, " \
@@ -63,8 +64,11 @@ class ENUM(object):
         self.candidates = candidates
 
     def match(self, value):
-        # type: (unicode) -> bool
-        return value in self.candidates
+        # type: (Union[unicode,List,Tuple]) -> bool
+        if isinstance(value, (list, tuple)):
+            return all(item in self.candidates for item in value)
+        else:
+            return value in self.candidates
 
 
 string_classes = [text_type]  # type: List
@@ -120,6 +124,7 @@ class Config(object):
         primary_domain = ('py', 'env', [NoneType]),
         needs_sphinx = (None, None, string_classes),
         needs_extensions = ({}, None),
+        manpages_url = (None, 'env'),
         nitpicky = (False, None),
         nitpick_ignore = ([], None),
         numfig = (False, 'env'),
@@ -132,6 +137,11 @@ class Config(object):
 
         tls_verify = (True, 'env'),
         tls_cacerts = (None, 'env'),
+        smartquotes = (True, 'env'),
+        smartquotes_action = ('qDe', 'env'),
+        smartquotes_excludes = ({'languages': ['ja'],
+                                 'builders': ['man', 'text']},
+                                'env'),
     )  # type: Dict[unicode, Tuple]
 
     def __init__(self, dirname, filename, overrides, tags):
@@ -152,6 +162,8 @@ class Config(object):
                     raise ConfigError(CONFIG_SYNTAX_ERROR % err)
                 except SystemExit:
                     raise ConfigError(CONFIG_EXIT_ERROR)
+                except Exception:
+                    raise ConfigError(CONFIG_ERROR % traceback.format_exc())
 
         self._raw_config = config
         # these two must be preinitialized because extensions can add their
@@ -233,8 +245,8 @@ class Config(object):
         else:
             defvalue = self.values[name][0]
             if isinstance(defvalue, dict):
-                raise ValueError(_('cannot override dictionary config setting %r, '
-                                   'ignoring (use %r to set individual elements)') %
+                raise ValueError(__('cannot override dictionary config setting %r, '
+                                    'ignoring (use %r to set individual elements)') %
                                  (name, name + '.key=value'))
             elif isinstance(defvalue, list):
                 return value.split(',')
@@ -242,13 +254,13 @@ class Config(object):
                 try:
                     return int(value)
                 except ValueError:
-                    raise ValueError(_('invalid number %r for config value %r, ignoring') %
+                    raise ValueError(__('invalid number %r for config value %r, ignoring') %
                                      (value, name))
             elif hasattr(defvalue, '__call__'):
                 return value
             elif defvalue is not None and not isinstance(defvalue, string_types):
-                raise ValueError(_('cannot override config setting %r with unsupported '
-                                   'type, ignoring') % name)
+                raise ValueError(__('cannot override config setting %r with unsupported '
+                                    'type, ignoring') % name)
             else:
                 return value
 
@@ -277,7 +289,8 @@ class Config(object):
                     config.setdefault(realvalname, {})[key] = value
                     continue
                 elif valname not in self.values:
-                    logger.warning(_('unknown config value %r in override, ignoring'), valname)
+                    logger.warning(__('unknown config value %r in override, ignoring'),
+                                   valname)
                     continue
                 if isinstance(value, string_types):
                     config[valname] = self.convert_overrides(valname, value)
@@ -287,7 +300,7 @@ class Config(object):
                 logger.warning("%s", exc)
         for name in config:
             if name in self.values:
-                self.__dict__[name] = config[name]
+                self.__dict__[name] = config[name]  # type: ignore
         if isinstance(self.source_suffix, string_types):  # type: ignore
             self.source_suffix = [self.source_suffix]  # type: ignore
 
@@ -296,7 +309,7 @@ class Config(object):
         if name.startswith('_'):
             raise AttributeError(name)
         if name not in self.values:
-            raise AttributeError(_('No such config value: %s') % name)
+            raise AttributeError(__('No such config value: %s') % name)
         default = self.values[name][0]
         if hasattr(default, '__call__'):
             return default(self)
@@ -328,5 +341,7 @@ class Config(object):
         self.values[name] = (default, rebuild, types)
 
     def filter(self, rebuild):
-        # type: (str) -> Iterator[ConfigValue]
-        return (value for value in self if value.rebuild == rebuild)  # type: ignore
+        # type: (Union[unicode, List[unicode]]) -> Iterator[ConfigValue]
+        if isinstance(rebuild, string_types):
+            rebuild = [rebuild]
+        return (value for value in self if value.rebuild in rebuild)  # type: ignore
