@@ -3,13 +3,14 @@
     sphinx.directives.code
     ~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import sys
 import codecs
+import sys
 from difflib import unified_diff
+from typing import TYPE_CHECKING
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
@@ -21,8 +22,7 @@ from sphinx.util import logging
 from sphinx.util import parselinenos
 from sphinx.util.nodes import set_source_info
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from typing import Any, Dict, List, Tuple  # NOQA
     from sphinx.application import Sphinx  # NOQA
     from sphinx.config import Config  # NOQA
@@ -256,7 +256,7 @@ class LiteralIncludeReader(object):
             else:
                 start = tags[pyobject][1]
                 end = tags[pyobject][2]
-                lines = lines[start - 1:end - 1]
+                lines = lines[start - 1:end]
                 if 'lineno-match' in self.options:
                     self.lineno_start = start
 
@@ -311,11 +311,11 @@ class LiteralIncludeReader(object):
                             self.lineno_start += lineno
 
                         return lines[lineno:]
+
+            if inclusive is True:
+                raise ValueError('start-after pattern not found: %s' % start)
             else:
-                if inclusive is True:
-                    raise ValueError('start-after pattern not found: %s' % start)
-                else:
-                    raise ValueError('start-at pattern not found: %s' % start)
+                raise ValueError('start-at pattern not found: %s' % start)
 
         return lines
 
@@ -340,11 +340,10 @@ class LiteralIncludeReader(object):
                             return []
                         else:
                             return lines[:lineno]
+            if inclusive is True:
+                raise ValueError('end-at pattern not found: %s' % end)
             else:
-                if inclusive is True:
-                    raise ValueError('end-at pattern not found: %s' % end)
-                else:
-                    raise ValueError('end-before pattern not found: %s' % end)
+                raise ValueError('end-before pattern not found: %s' % end)
 
         return lines
 
