@@ -5,27 +5,37 @@
 
     Allow reference sections by :ref: role using its title.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
+from typing import TYPE_CHECKING
+
 from docutils import nodes
+
 from sphinx.util import logging
 from sphinx.util.nodes import clean_astext
 
 logger = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+    # For type annotation
+    from typing import Any, Dict  # NOQA
+    from sphinx.application import Sphinx  # NOQA
+
 
 def register_sections_as_label(app, document):
+    # type: (Sphinx, nodes.Node) -> None
     labels = app.env.domaindata['std']['labels']
     anonlabels = app.env.domaindata['std']['anonlabels']
     for node in document.traverse(nodes.section):
         labelid = node['ids'][0]
         docname = app.env.docname
+        ref_name = getattr(node[0], 'rawsource', node[0].astext())
         if app.config.autosectionlabel_prefix_document:
-            name = nodes.fully_normalize_name(docname + ':' + node[0].astext())
+            name = nodes.fully_normalize_name(docname + ':' + ref_name)
         else:
-            name = nodes.fully_normalize_name(node[0].astext())
+            name = nodes.fully_normalize_name(ref_name)
         sectname = clean_astext(node[0])
 
         if name in labels:
@@ -38,5 +48,12 @@ def register_sections_as_label(app, document):
 
 
 def setup(app):
+    # type: (Sphinx) -> Dict[unicode, Any]
     app.add_config_value('autosectionlabel_prefix_document', False, 'env')
     app.connect('doctree-read', register_sections_as_label)
+
+    return {
+        'version': 'builtin',
+        'parallel_read_safe': True,
+        'parallel_write_safe': True,
+    }
