@@ -5,14 +5,20 @@
 
     Utilities for docstring processing.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
+from __future__ import absolute_import
 
 import sys
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from typing import List  # NOQA
 
 
 def prepare_docstring(s, ignore=1):
+    # type: (unicode, int) -> List[unicode]
     """Convert a docstring into lines of parseable reST.  Remove common leading
     indentation, where the indentation of a given number of lines (usually just
     one) is ignored.
@@ -46,6 +52,7 @@ def prepare_docstring(s, ignore=1):
 
 
 def prepare_commentdoc(s):
+    # type: (unicode) -> List[unicode]
     """Extract documentation comment lines (starting with #:) and return them
     as a list of lines.  Returns an empty list if there is no documentation.
     """
