@@ -5,30 +5,31 @@
 
     Logging utility functions for Sphinx.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import absolute_import
 
 import logging
 import logging.handlers
-from contextlib import contextmanager
 from collections import defaultdict
+from contextlib import contextmanager
+from typing import TYPE_CHECKING
 
-from six import PY2, StringIO
 from docutils import nodes
 from docutils.utils import get_source_line
+from six import PY2, StringIO
 
 from sphinx.errors import SphinxWarning
 from sphinx.util.console import colorize
 
-if False:
-    # For type annotation
+if TYPE_CHECKING:
     from typing import Any, Dict, Generator, IO, List, Tuple, Union  # NOQA
     from docutils import nodes  # NOQA
     from sphinx.application import Sphinx  # NOQA
 
 
+NAMESPACE = 'sphinx'
 VERBOSE = 15
 
 LEVEL_NAMES = defaultdict(lambda: logging.WARNING)  # type: Dict[str, int]
@@ -51,15 +52,26 @@ VERBOSITY_MAP.update({
 
 COLOR_MAP = defaultdict(lambda: 'blue')  # type: Dict[int, unicode]
 COLOR_MAP.update({
-    logging.WARNING: 'darkred',
+    logging.ERROR: 'darkred',
+    logging.WARNING: 'red',
     logging.DEBUG: 'darkgray',
 })
 
 
 def getLogger(name):
     # type: (str) -> SphinxLoggerAdapter
-    """Get logger wrapped by SphinxLoggerAdapter."""
-    return SphinxLoggerAdapter(logging.getLogger(name), {})
+    """Get logger wrapped by SphinxLoggerAdapter.
+
+    Sphinx logger always uses ``sphinx.*`` namesapce to be independent from
+    settings of root logger.  It enables to log stably even if 3rd party
+    extension or imported application resets logger settings.
+    """
+    # add sphinx prefix to name forcely
+    logger = logging.getLogger(NAMESPACE + '.' + name)
+    # Forcely enable logger
+    logger.disabled = False
+    # wrap logger by SphinxLoggerAdapter
+    return SphinxLoggerAdapter(logger, {})
 
 
 def convert_serializable(records):
@@ -70,6 +82,10 @@ def convert_serializable(records):
         r.msg = r.getMessage()
         r.args = ()
 
+        location = getattr(r, 'location', None)
+        if isinstance(location, nodes.Node):
+            r.location = get_node_location(location)  # type: ignore
+
 
 class SphinxWarningLogRecord(logging.LogRecord):
     """Log record class supporting location"""
@@ -135,7 +151,7 @@ class NewLineStreamHandlerPY2(logging.StreamHandler):
         # type: (logging.LogRecord) -> None
         try:
             self.acquire()
-            stream = self.stream  # type: ignore
+            stream = self.stream
             if getattr(record, 'nonl', False):
                 # remove return code forcely when nonl=True
                 self.stream = StringIO()
@@ -202,7 +218,7 @@ class MemoryHandler(logging.handlers.BufferingHandler):
 def pending_warnings():
     # type: () -> Generator
     """contextmanager to pend logging warnings temporary."""
-    logger = logging.getLogger()
+    logger = logging.getLogger(NAMESPACE)
     memhandler = MemoryHandler()
     memhandler.setLevel(logging.WARNING)
 
@@ -228,7 +244,7 @@ def pending_warnings():
 def pending_logging():
     # type: () -> Generator
     """contextmanager to pend logging all logs temporary."""
-    logger = logging.getLogger()
+    logger = logging.getLogger(NAMESPACE)
     memhandler = MemoryHandler()
 
     try:
@@ -248,6 +264,27 @@ def pending_logging():
         memhandler.flushTo(logger)
 
 
+@contextmanager
+def skip_warningiserror(skip=True):
+    # type: (bool) -> Generator
+    """contextmanager to skip WarningIsErrorFilter for a while."""
+    logger = logging.getLogger(NAMESPACE)
+
+    if skip is False:
+        yield
+    else:
+        try:
+            disabler = DisableWarningIsErrorFilter()
+            for handler in logger.handlers:
+                # use internal method; filters.insert() directly to install disabler
+                # before WarningIsErrorFilter
+                handler.filters.insert(0, disabler)
+            yield
+        finally:
+            for handler in logger.handlers:
+                handler.removeFilter(disabler)
+
+
 class LogCollector(object):
     def __init__(self):
         # type: () -> None
@@ -306,7 +343,13 @@ class WarningSuppressor(logging.Filter):
         type = getattr(record, 'type', None)
         subtype = getattr(record, 'subtype', None)
 
-        if is_suppressed_warning(type, subtype, self.app.config.suppress_warnings):
+        try:
+            suppress_warnings = self.app.config.suppress_warnings
+        except AttributeError:
+            # config is not initialized yet (ex. in conf.py)
+            suppress_warnings = []
+
+        if is_suppressed_warning(type, subtype, suppress_warnings):
             return False
         else:
             self.app._warncount += 1
@@ -323,16 +366,33 @@ class WarningIsErrorFilter(logging.Filter):
 
     def filter(self, record):
         # type: (logging.LogRecord) -> bool
-        if self.app.warningiserror:
+        if getattr(record, 'skip_warningsiserror', False):
+            # disabled by DisableWarningIsErrorFilter
+            return True
+        elif self.app.warningiserror:
             location = getattr(record, 'location', '')
+            try:
+                message = record.msg % record.args
+            except TypeError:
+                message = record.msg  # use record.msg itself
+
             if location:
-                raise SphinxWarning(location + ":" + record.msg % record.args)
+                raise SphinxWarning(location + ":" + message)
             else:
-                raise SphinxWarning(record.msg % record.args)
+                raise SphinxWarning(message)
         else:
             return True
 
 
+class DisableWarningIsErrorFilter(logging.Filter):
+    """Disable WarningIsErrorFilter if this filter installed."""
+
+    def filter(self, record):
+        # type: (logging.LogRecord) -> bool
+        record.skip_warningsiserror = True  # type: ignore
+        return True
+
+
 class WarningLogRecordTranslator(logging.Filter):
     """Converts a log record to one Sphinx expects
 
@@ -359,21 +419,26 @@ class WarningLogRecordTranslator(logging.Filter):
             else:
                 record.location = None
         elif isinstance(location, nodes.Node):
-            (source, line) = get_source_line(location)
-            if source and line:
-                record.location = "%s:%s" % (source, line)
-            elif source:
-                record.location = "%s:" % source
-            elif line:
-                record.location = "<unknown>:%s" % line
-            else:
-                record.location = None
+            record.location = get_node_location(location)
         elif location and ':' not in location:
             record.location = '%s' % self.app.env.doc2path(location)
 
         return True
 
 
+def get_node_location(node):
+    # type: (nodes.Node) -> str
+    (source, line) = get_source_line(node)
+    if source and line:
+        return "%s:%s" % (source, line)
+    elif source:
+        return "%s:" % source
+    elif line:
+        return "<unknown>:%s" % line
+    else:
+        return None
+
+
 class ColorizeFormatter(logging.Formatter):
     def format(self, record):
         # type: (logging.LogRecord) -> str
@@ -424,8 +489,9 @@ class LastMessagesWriter(object):
 def setup(app, status, warning):
     # type: (Sphinx, IO, IO) -> None
     """Setup root logger for Sphinx"""
-    logger = logging.getLogger()
-    logger.setLevel(logging.NOTSET)
+    logger = logging.getLogger(NAMESPACE)
+    logger.setLevel(logging.DEBUG)
+    logger.propagate = False
 
     # clear all handlers
     for handler in logger.handlers[:]:
