@@ -6,18 +6,21 @@
     Implements the low-level algorithms Sphinx uses for the versioning of
     doctrees.
 
-    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
-from uuid import uuid4
-from operator import itemgetter
 from itertools import product
+from operator import itemgetter
+from typing import TYPE_CHECKING
+from uuid import uuid4
 
 from six import iteritems
+from six.moves import cPickle as pickle
 from six.moves import range, zip_longest
 
-if False:
-    # For type annotation
+from sphinx.transforms import SphinxTransform
+
+if TYPE_CHECKING:
     from typing import Any, Iterator  # NOQA
     from docutils import nodes  # NOQA
 
@@ -148,3 +151,32 @@ def levenshtein_distance(a, b):
             current_row.append(min(insertions, deletions, substitutions))
         previous_row = current_row  # type: ignore
     return previous_row[-1]
+
+
+class UIDTransform(SphinxTransform):
+    """Add UIDs to doctree for versioning."""
+    default_priority = 100
+
+    def apply(self):
+        env = self.env
+        old_doctree = None
+        if env.versioning_compare:
+            # get old doctree
+            try:
+                filename = env.doc2path(env.docname, env.doctreedir, '.doctree')
+                with open(filename, 'rb') as f:
+                    old_doctree = pickle.load(f)
+            except EnvironmentError:
+                pass
+
+        # add uids for versioning
+        if not env.versioning_compare or old_doctree is None:
+            list(add_uids(self.document, env.versioning_condition))
+        else:
+            list(merge_doctrees(old_doctree, self.document, env.versioning_condition))
+
+
+def prepare(document):
+    """Simple wrapper for UIDTransform."""
+    transform = UIDTransform(document)
+    transform.apply()
