@@ -5,25 +5,31 @@
 
     docutils writers handling Sphinx' custom nodes.
 
-    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2018 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import sys
-import posixpath
-import os
 import copy
-import warnings
+import os
+import posixpath
+import sys
+from typing import TYPE_CHECKING
 
-from six import string_types
 from docutils import nodes
 from docutils.writers.html4css1 import Writer, HTMLTranslator as BaseTranslator
+from six import string_types
 
 from sphinx import addnodes
-from sphinx.deprecation import RemovedInSphinx16Warning
 from sphinx.locale import admonitionlabels, _
+from sphinx.util import logging
 from sphinx.util.images import get_image_size
-from sphinx.util.smartypants import sphinx_smarty_pants
+
+if TYPE_CHECKING:
+    from typing import Any  # NOQA
+    from sphinx.builders.html import StandaloneHTMLBuilder  # NOQA
+
+
+logger = logging.getLogger(__name__)
 
 # A good overview of the purpose behind these classes can be found here:
 # http://www.arnebrodowski.de/blog/write-your-own-restructuredtext-writer.html
@@ -38,13 +44,15 @@ class HTMLWriter(Writer):
             _setting[2]['default'] = 0
 
     def __init__(self, builder):
+        # type: (StandaloneHTMLBuilder) -> None
         Writer.__init__(self)
         self.builder = builder
 
     def translate(self):
+        # type: () -> None
         # sadly, this is mostly copied from parent class
-        self.visitor = visitor = self.builder.translator_class(self.builder,
-                                                               self.document)
+        self.visitor = visitor = self.builder.create_translator(self.builder,
+                                                                self.document)
         self.document.walkabout(visitor)
         self.output = visitor.astext()
         for attr in ('head_prefix', 'stylesheet', 'head', 'body_prefix',
@@ -62,15 +70,16 @@ class HTMLTranslator(BaseTranslator):
     """
 
     def __init__(self, builder, *args, **kwds):
+        # type: (StandaloneHTMLBuilder, Any, Any) -> None
         BaseTranslator.__init__(self, *args, **kwds)
         self.highlighter = builder.highlighter
-        self.no_smarty = 0
         self.builder = builder
         self.highlightlang = self.highlightlang_base = \
             builder.config.highlight_language
         self.highlightopts = builder.config.highlight_options
         self.highlightlinenothreshold = sys.maxsize
         self.docnames = [builder.current_docname]  # for singlehtml builder
+        self.manpages_url = builder.config.manpages_url
         self.protect_literal_text = 0
         self.permalink_text = builder.config.html_add_permalinks
         # support backwards-compatible setting to a bool
@@ -81,22 +90,29 @@ class HTMLTranslator(BaseTranslator):
         self.param_separator = ''
         self.optional_param_level = 0
         self._table_row_index = 0
+        self._fieldlist_row_index = 0
+        self.required_params_left = 0
 
     def visit_start_of_file(self, node):
+        # type: (nodes.Node) -> None
         # only occurs in the single-file builder
         self.docnames.append(node['docname'])
         self.body.append('<span id="document-%s"></span>' % node['docname'])
 
     def depart_start_of_file(self, node):
+        # type: (nodes.Node) -> None
         self.docnames.pop()
 
     def visit_desc(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'dl', CLASS=node['objtype']))
 
     def depart_desc(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</dl>\n\n')
 
     def visit_desc_signature(self, node):
+        # type: (nodes.Node) -> None
         # the id is set automatically
         self.body.append(self.starttag(node, 'dt'))
         # anchor for per-desc interactive data
@@ -105,44 +121,56 @@ class HTMLTranslator(BaseTranslator):
             self.body.append('<!--[%s]-->' % node['ids'][0])
 
     def depart_desc_signature(self, node):
+        # type: (nodes.Node) -> None
         if not node.get('is_multiline'):
             self.add_permalink_ref(node, _('Permalink to this definition'))
         self.body.append('</dt>\n')
 
     def visit_desc_signature_line(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def depart_desc_signature_line(self, node):
+        # type: (nodes.Node) -> None
         if node.get('add_permalink'):
             # the permalink info is on the parent desc_signature node
             self.add_permalink_ref(node.parent, _('Permalink to this definition'))
         self.body.append('<br />')
 
     def visit_desc_addname(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'code', '', CLASS='descclassname'))
 
     def depart_desc_addname(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</code>')
 
     def visit_desc_type(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def depart_desc_type(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_desc_returns(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(' &#x2192; ')
 
     def depart_desc_returns(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_desc_name(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'code', '', CLASS='descname'))
 
     def depart_desc_name(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</code>')
 
     def visit_desc_parameterlist(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('<span class="sig-paren">(</span>')
         self.first_param = 1
         self.optional_param_level = 0
@@ -152,6 +180,7 @@ class HTMLTranslator(BaseTranslator):
         self.param_separator = node.child_text_separator
 
     def depart_desc_parameterlist(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('<span class="sig-paren">)</span>')
 
     # If required parameters are still to come, then put the comma after
@@ -161,6 +190,7 @@ class HTMLTranslator(BaseTranslator):
     #     foo([a, ]b, c[, d])
     #
     def visit_desc_parameter(self, node):
+        # type: (nodes.Node) -> None
         if self.first_param:
             self.first_param = 0
         elif not self.required_params_left:
@@ -171,39 +201,49 @@ class HTMLTranslator(BaseTranslator):
             self.body.append('<em>')
 
     def depart_desc_parameter(self, node):
+        # type: (nodes.Node) -> None
         if not node.hasattr('noemph'):
             self.body.append('</em>')
         if self.required_params_left:
             self.body.append(self.param_separator)
 
     def visit_desc_optional(self, node):
+        # type: (nodes.Node) -> None
         self.optional_param_level += 1
         self.body.append('<span class="optional">[</span>')
 
     def depart_desc_optional(self, node):
+        # type: (nodes.Node) -> None
         self.optional_param_level -= 1
         self.body.append('<span class="optional">]</span>')
 
     def visit_desc_annotation(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'em', '', CLASS='property'))
 
     def depart_desc_annotation(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</em>')
 
     def visit_desc_content(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'dd', ''))
 
     def depart_desc_content(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</dd>')
 
     def visit_versionmodified(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'div', CLASS=node['type']))
 
     def depart_versionmodified(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</div>\n')
 
     # overwritten
     def visit_reference(self, node):
+        # type: (nodes.Node) -> None
         atts = {'class': 'reference'}
         if node.get('internal') or 'refuri' not in node:
             atts['class'] += ' internal'
@@ -233,17 +273,21 @@ class HTMLTranslator(BaseTranslator):
                              '.'.join(map(str, node['secnumber'])))
 
     def visit_number_reference(self, node):
+        # type: (nodes.Node) -> None
         self.visit_reference(node)
 
     def depart_number_reference(self, node):
+        # type: (nodes.Node) -> None
         self.depart_reference(node)
 
     # overwritten -- we don't want source comments to show up in the HTML
     def visit_comment(self, node):
+        # type: (nodes.Node) -> None
         raise nodes.SkipNode
 
     # overwritten
     def visit_admonition(self, node, name=''):
+        # type: (nodes.Node, unicode) -> None
         self.body.append(self.starttag(
             node, 'div', CLASS=('admonition ' + name)))
         if name:
@@ -251,23 +295,24 @@ class HTMLTranslator(BaseTranslator):
         self.set_first_last(node)
 
     def visit_seealso(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'seealso')
 
     def depart_seealso(self, node):
+        # type: (nodes.Node) -> None
         self.depart_admonition(node)
 
     def add_secnumber(self, node):
+        # type: (nodes.Node) -> None
         if node.get('secnumber'):
             self.body.append('.'.join(map(str, node['secnumber'])) +
                              self.secnumber_suffix)
         elif isinstance(node.parent, nodes.section):
             if self.builder.name == 'singlehtml':
                 docname = self.docnames[-1]
-                anchorname = '#' + node.parent['ids'][0]
-                if (docname, anchorname) not in self.builder.secnumbers:
-                    anchorname = (docname, '')  # try first heading which has no anchor
-                else:
-                    anchorname = (docname, anchorname)
+                anchorname = "%s/#%s" % (docname, node.parent['ids'][0])
+                if anchorname not in self.builder.secnumbers:
+                    anchorname = "%s/" % docname  # try first heading which has no anchor
             else:
                 anchorname = '#' + node.parent['ids'][0]
                 if anchorname not in self.builder.secnumbers:
@@ -278,9 +323,11 @@ class HTMLTranslator(BaseTranslator):
                                  self.secnumber_suffix)
 
     def add_fignumber(self, node):
+        # type: (nodes.Node) -> None
         def append_fignumber(figtype, figure_id):
+            # type: (unicode, unicode) -> None
             if self.builder.name == 'singlehtml':
-                key = (self.docnames[-1], figtype)
+                key = u"%s/%s" % (self.docnames[-1], figtype)
             else:
                 key = figtype
 
@@ -289,26 +336,28 @@ class HTMLTranslator(BaseTranslator):
                 prefix = self.builder.config.numfig_format.get(figtype)
                 if prefix is None:
                     msg = 'numfig_format is not defined for %s' % figtype
-                    self.builder.warn(msg)
+                    logger.warning(msg)
                 else:
                     numbers = self.builder.fignumbers[key][figure_id]
                     self.body.append(prefix % '.'.join(map(str, numbers)) + ' ')
                     self.body.append('</span>')
 
-        figtype = self.builder.env.domains['std'].get_figtype(node)
+        figtype = self.builder.env.domains['std'].get_figtype(node)  # type: ignore
         if figtype:
             if len(node['ids']) == 0:
                 msg = 'Any IDs not assigned for %s node' % node.tagname
-                self.builder.env.warn_node(msg, node)
+                logger.warning(msg, location=node)
             else:
                 append_fignumber(figtype, node['ids'][0])
 
     def add_permalink_ref(self, node, title):
+        # type: (nodes.Node, unicode) -> None
         if node['ids'] and self.permalink_text and self.builder.add_permalinks:
             format = u'<a class="headerlink" href="#%s" title="%s">%s</a>'
             self.body.append(format % (node['ids'][0], title, self.permalink_text))
 
     def generate_targets_for_listing(self, node):
+        # type: (nodes.Node) -> None
         """Generate hyperlink targets for listings.
 
         Original visit_bullet_list(), visit_definition_list() and visit_enumerated_list()
@@ -324,6 +373,7 @@ class HTMLTranslator(BaseTranslator):
 
     # overwritten
     def visit_bullet_list(self, node):
+        # type: (nodes.Node) -> None
         if len(node) == 1 and node[0].tagname == 'toctree':
             # avoid emitting empty <ul></ul>
             raise nodes.SkipNode
@@ -332,11 +382,13 @@ class HTMLTranslator(BaseTranslator):
 
     # overwritten
     def visit_enumerated_list(self, node):
+        # type: (nodes.Node) -> None
         self.generate_targets_for_listing(node)
         BaseTranslator.visit_enumerated_list(self, node)
 
     # overwritten
     def visit_title(self, node):
+        # type: (nodes.Node) -> None
         BaseTranslator.visit_title(self, node)
         self.add_secnumber(node)
         self.add_fignumber(node.parent)
@@ -344,6 +396,7 @@ class HTMLTranslator(BaseTranslator):
             self.body.append('<span class="caption-text">')
 
     def depart_title(self, node):
+        # type: (nodes.Node) -> None
         close_tag = self.context[-1]
         if (self.permalink_text and self.builder.add_permalinks and
            node.parent.hasattr('ids') and node.parent['ids']):
@@ -366,6 +419,7 @@ class HTMLTranslator(BaseTranslator):
 
     # overwritten
     def visit_literal_block(self, node):
+        # type: (nodes.Node) -> None
         if node.rawsource != node.astext():
             # most probably a parsed-literal block -- don't highlight
             return BaseTranslator.visit_literal_block(self, node)
@@ -385,17 +439,17 @@ class HTMLTranslator(BaseTranslator):
         else:
             opts = {}
 
-        def warner(msg, **kwargs):
-            self.builder.warn(msg, (self.builder.current_docname, node.line), **kwargs)
         highlighted = self.highlighter.highlight_block(
-            node.rawsource, lang, opts=opts, warn=warner, linenos=linenos,
-            **highlight_args)
+            node.rawsource, lang, opts=opts, linenos=linenos,
+            location=(self.builder.current_docname, node.line), **highlight_args
+        )
         starttag = self.starttag(node, 'div', suffix='',
-                                 CLASS='highlight-%s' % lang)
+                                 CLASS='highlight-%s notranslate' % lang)
         self.body.append(starttag + highlighted + '</div>\n')
         raise nodes.SkipNode
 
     def visit_caption(self, node):
+        # type: (nodes.Node) -> None
         if isinstance(node.parent, nodes.container) and node.parent.get('literal_block'):
             self.body.append('<div class="code-block-caption">')
         else:
@@ -404,6 +458,7 @@ class HTMLTranslator(BaseTranslator):
         self.body.append(self.starttag(node, 'span', '', CLASS='caption-text'))
 
     def depart_caption(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</span>')
 
         # append permalink if available
@@ -422,26 +477,39 @@ class HTMLTranslator(BaseTranslator):
             BaseTranslator.depart_caption(self, node)
 
     def visit_doctest_block(self, node):
+        # type: (nodes.Node) -> None
         self.visit_literal_block(node)
 
     # overwritten to add the <div> (for XHTML compliance)
     def visit_block_quote(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'blockquote') + '<div>')
 
     def depart_block_quote(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</div></blockquote>\n')
 
     # overwritten
     def visit_literal(self, node):
-        self.body.append(self.starttag(node, 'code', '',
-                                       CLASS='docutils literal'))
-        self.protect_literal_text += 1
+        # type: (nodes.Node) -> None
+        if 'kbd' in node['classes']:
+            self.body.append(self.starttag(node, 'kbd', '',
+                                           CLASS='docutils literal notranslate'))
+        else:
+            self.body.append(self.starttag(node, 'code', '',
+                                           CLASS='docutils literal notranslate'))
+            self.protect_literal_text += 1
 
     def depart_literal(self, node):
-        self.protect_literal_text -= 1
-        self.body.append('</code>')
+        # type: (nodes.Node) -> None
+        if 'kbd' in node['classes']:
+            self.body.append('</kbd>')
+        else:
+            self.protect_literal_text -= 1
+            self.body.append('</code>')
 
     def visit_productionlist(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'pre'))
         names = []
         for production in node:
@@ -461,23 +529,29 @@ class HTMLTranslator(BaseTranslator):
         raise nodes.SkipNode
 
     def depart_productionlist(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_production(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def depart_production(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_centered(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.starttag(node, 'p', CLASS="centered") +
                          '<strong>')
 
     def depart_centered(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</strong></p>')
 
     # overwritten
     def should_be_compact_paragraph(self, node):
+        # type: (nodes.Node) -> bool
         """Determine if the <p> tags around paragraph can be omitted."""
         if isinstance(node.parent, addnodes.desc_content):
             # Never compact desc_content items.
@@ -488,19 +562,24 @@ class HTMLTranslator(BaseTranslator):
         return BaseTranslator.should_be_compact_paragraph(self, node)
 
     def visit_compact_paragraph(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def depart_compact_paragraph(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_highlightlang(self, node):
+        # type: (nodes.Node) -> None
         self.highlightlang = node['lang']
         self.highlightlinenothreshold = node['linenothreshold']
 
     def depart_highlightlang(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_download_reference(self, node):
+        # type: (nodes.Node) -> None
         if self.builder.download_support and node.hasattr('filename'):
             self.body.append(
                 '<a class="reference download internal" href="%s" download="">' %
@@ -510,10 +589,12 @@ class HTMLTranslator(BaseTranslator):
             self.context.append('')
 
     def depart_download_reference(self, node):
+        # type: (nodes.Node) -> None
         self.body.append(self.context.pop())
 
     # overwritten
     def visit_image(self, node):
+        # type: (nodes.Node) -> None
         olduri = node['uri']
         # rewrite the URI if the environment knows about it
         if olduri in self.builder.images:
@@ -544,8 +625,8 @@ class HTMLTranslator(BaseTranslator):
             if not ('width' in node and 'height' in node):
                 size = get_image_size(os.path.join(self.builder.srcdir, olduri))
                 if size is None:
-                    self.builder.env.warn_node('Could not obtain image size. '
-                                               ':scale: option is ignored.', node)
+                    logger.warning('Could not obtain image size. :scale: option is ignored.',
+                                   location=node)
                 else:
                     if 'width' not in node:
                         node['width'] = str(size[0])
@@ -555,55 +636,65 @@ class HTMLTranslator(BaseTranslator):
 
     # overwritten
     def depart_image(self, node):
+        # type: (nodes.Node) -> None
         if node['uri'].lower().endswith(('svg', 'svgz')):
             self.body.append(self.context.pop())
         else:
             BaseTranslator.depart_image(self, node)
 
     def visit_toctree(self, node):
+        # type: (nodes.Node) -> None
         # this only happens when formatting a toc from env.tocs -- in this
         # case we don't want to include the subtree
         raise nodes.SkipNode
 
     def visit_index(self, node):
+        # type: (nodes.Node) -> None
         raise nodes.SkipNode
 
     def visit_tabular_col_spec(self, node):
+        # type: (nodes.Node) -> None
         raise nodes.SkipNode
 
     def visit_glossary(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def depart_glossary(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_acks(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def depart_acks(self, node):
+        # type: (nodes.Node) -> None
         pass
 
     def visit_hlist(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('<table class="hlist"><tr>')
 
     def depart_hlist(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</tr></table>\n')
 
     def visit_hlistcol(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('<td>')
 
     def depart_hlistcol(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</td>')
 
     def visit_option_group(self, node):
         BaseTranslator.visit_option_group(self, node)
         self.context[-2] = self.context[-2].replace('&nbsp;', '&#160;')
 
-    def bulk_text_processor(self, text):
-        return text
-
     # overwritten
     def visit_Text(self, node):
+        # type: (nodes.Node) -> None
         text = node.astext()
         encoded = self.encode(text)
         if self.protect_literal_text:
@@ -622,107 +713,129 @@ class HTMLTranslator(BaseTranslator):
         else:
             if self.in_mailto and self.settings.cloak_email_addresses:
                 encoded = self.cloak_email(encoded)
-            else:
-                encoded = self.bulk_text_processor(encoded)
             self.body.append(encoded)
 
     def visit_note(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'note')
 
     def depart_note(self, node):
+        # type: (nodes.Node) -> None
         self.depart_admonition(node)
 
     def visit_warning(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'warning')
 
     def depart_warning(self, node):
+        # type: (nodes.Node) -> None
         self.depart_admonition(node)
 
     def visit_attention(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'attention')
 
     def depart_attention(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
     def visit_caution(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'caution')
 
     def depart_caution(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
     def visit_danger(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'danger')
 
     def depart_danger(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
     def visit_error(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'error')
 
     def depart_error(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
     def visit_hint(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'hint')
 
     def depart_hint(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
     def visit_important(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'important')
 
     def depart_important(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
     def visit_tip(self, node):
+        # type: (nodes.Node) -> None
         self.visit_admonition(node, 'tip')
 
     def depart_tip(self, node):
-        self.depart_admonition()
+        # type: (nodes.Node) -> None
+        self.depart_admonition(node)
 
-    # these are only handled specially in the SmartyPantsHTMLTranslator
     def visit_literal_emphasis(self, node):
+        # type: (nodes.Node) -> None
         return self.visit_emphasis(node)
 
     def depart_literal_emphasis(self, node):
+        # type: (nodes.Node) -> None
         return self.depart_emphasis(node)
 
     def visit_literal_strong(self, node):
+        # type: (nodes.Node) -> None
         return self.visit_strong(node)
 
     def depart_literal_strong(self, node):
+        # type: (nodes.Node) -> None
         return self.depart_strong(node)
 
     def visit_abbreviation(self, node):
+        # type: (nodes.Node) -> None
         attrs = {}
         if node.hasattr('explanation'):
             attrs['title'] = node['explanation']
         self.body.append(self.starttag(node, 'abbr', '', **attrs))
 
     def depart_abbreviation(self, node):
+        # type: (nodes.Node) -> None
         self.body.append('</abbr>')
 
-    def visit_termsep(self, node):
-        warnings.warn('sphinx.addnodes.termsep will be removed at Sphinx-1.6. '
-                      'This warning is displayed because some Sphinx extension '
-                      'uses sphinx.addnodes.termsep. Please report it to '
-                      'author of the extension.', RemovedInSphinx16Warning)
-        self.body.append('<br />')
-        raise nodes.SkipNode
-
     def visit_manpage(self, node):
-        return self.visit_literal_emphasis(node)
+        # type: (nodes.Node) -> None
+        self.visit_literal_emphasis(node)
+        if self.manpages_url:
+            node['refuri'] = self.manpages_url.format(**node.attributes)
+            self.visit_reference(node)
 
     def depart_manpage(self, node):
-        return self.depart_literal_emphasis(node)
+        # type: (nodes.Node) -> None
+        if self.manpages_url:
+            self.depart_reference(node)
+        self.depart_literal_emphasis(node)
 
     # overwritten to add even/odd classes
 
     def visit_table(self, node):
+        # type: (nodes.Node) -> None
         self._table_row_index = 0
         return BaseTranslator.visit_table(self, node)
 
     def visit_row(self, node):
+        # type: (nodes.Node) -> None
         self._table_row_index += 1
         if self._table_row_index % 2 == 0:
             node['classes'].append('row-even')
@@ -737,10 +850,12 @@ class HTMLTranslator(BaseTranslator):
             self.body[-1] = '&#160;'
 
     def visit_field_list(self, node):
+        # type: (nodes.Node) -> None
         self._fieldlist_row_index = 0
         return BaseTranslator.visit_field_list(self, node)
 
     def visit_field(self, node):
+        # type: (nodes.Node) -> None
         self._fieldlist_row_index += 1
         if self._fieldlist_row_index % 2 == 0:
             node['classes'].append('field-even')
@@ -755,88 +870,13 @@ class HTMLTranslator(BaseTranslator):
             self.context[-1] = self.context[-1].replace('&nbsp;', '&#160;')
 
     def visit_math(self, node, math_env=''):
-        self.builder.warn('using "math" markup without a Sphinx math extension '
-                          'active, please use one of the math extensions '
-                          'described at http://sphinx-doc.org/ext/math.html',
-                          (self.builder.current_docname, node.line))
+        # type: (nodes.Node, unicode) -> None
+        logger.warning('using "math" markup without a Sphinx math extension '
+                       'active, please use one of the math extensions '
+                       'described at http://sphinx-doc.org/ext/math.html',
+                       location=(self.builder.current_docname, node.line))
         raise nodes.SkipNode
 
     def unknown_visit(self, node):
+        # type: (nodes.Node) -> None
         raise NotImplementedError('Unknown node: ' + node.__class__.__name__)
-
-
-class SmartyPantsHTMLTranslator(HTMLTranslator):
-    """
-    Handle ordinary text via smartypants, converting quotes and dashes
-    to the correct entities.
-    """
-
-    def __init__(self, *args, **kwds):
-        self.no_smarty = 0
-        HTMLTranslator.__init__(self, *args, **kwds)
-
-    def visit_literal(self, node):
-        self.no_smarty += 1
-        try:
-            # this raises SkipNode
-            HTMLTranslator.visit_literal(self, node)
-        finally:
-            self.no_smarty -= 1
-
-    def visit_literal_block(self, node):
-        self.no_smarty += 1
-        try:
-            HTMLTranslator.visit_literal_block(self, node)
-        except nodes.SkipNode:
-            # HTMLTranslator raises SkipNode for simple literal blocks,
-            # but not for parsed literal blocks
-            self.no_smarty -= 1
-            raise
-
-    def depart_literal_block(self, node):
-        HTMLTranslator.depart_literal_block(self, node)
-        self.no_smarty -= 1
-
-    def visit_literal_emphasis(self, node):
-        self.no_smarty += 1
-        self.visit_emphasis(node)
-
-    def depart_literal_emphasis(self, node):
-        self.depart_emphasis(node)
-        self.no_smarty -= 1
-
-    def visit_literal_strong(self, node):
-        self.no_smarty += 1
-        self.visit_strong(node)
-
-    def depart_literal_strong(self, node):
-        self.depart_strong(node)
-        self.no_smarty -= 1
-
-    def visit_desc_signature(self, node):
-        self.no_smarty += 1
-        HTMLTranslator.visit_desc_signature(self, node)
-
-    def depart_desc_signature(self, node):
-        self.no_smarty -= 1
-        HTMLTranslator.depart_desc_signature(self, node)
-
-    def visit_productionlist(self, node):
-        self.no_smarty += 1
-        try:
-            HTMLTranslator.visit_productionlist(self, node)
-        finally:
-            self.no_smarty -= 1
-
-    def visit_option(self, node):
-        self.no_smarty += 1
-        HTMLTranslator.visit_option(self, node)
-
-    def depart_option(self, node):
-        self.no_smarty -= 1
-        HTMLTranslator.depart_option(self, node)
-
-    def bulk_text_processor(self, text):
-        if self.no_smarty <= 0:
-            return sphinx_smarty_pants(text)
-        return text
